'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = dumpErrorBuffer;

var _object = require('lodash/object');

var _function = require('lodash/function');

var _collection = require('lodash/collection');

var _npmlog = require('npmlog');

var _npmlog2 = _interopRequireDefault(_npmlog);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _errorBuffer = require('contentful-batch-libs/utils/error-buffer');

var _errorBuffer2 = _interopRequireDefault(_errorBuffer);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function dumpErrorBuffer(params) {
  var message = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'Additional errors were found';
  var destinationSpace = params.destinationSpace,
      sourceSpace = params.sourceSpace,
      errorLogFile = params.errorLogFile;

  var loggedErrors = _errorBuffer2.default.drain();
  if (loggedErrors.length > 0) {
    var errorOutput = {
      additionalInfo: {}
    };
    errorOutput.errors = loggedErrors.map((0, _function.partialRight)(logErrorsWithAppLinks, sourceSpace, destinationSpace));
    var unresolvedLinks = additionalInfoForUnresolvedLinks(errorOutput.errors);
    if (unresolvedLinks) {
      errorOutput.additionalInfo.unresolvedLinks = unresolvedLinks;
    }
    _fs2.default.writeFileSync(errorLogFile, JSON.stringify(errorOutput, null, '  '));
    _npmlog2.default.warn(message);
    _npmlog2.default.warn('Check ' + errorLogFile + ' for details.');
  }
}

function logErrorsWithAppLinks(err, idx, loggedErrors, sourceSpace, destinationSpace) {
  var parsedError = JSON.parse(err.message);
  var requestUri = (0, _object.get)(parsedError, 'request.url');
  if (requestUri) {
    parsedError.webAppUrl = parseEntityUrl(sourceSpace, destinationSpace, requestUri);
  }
  return parsedError;
}

function parseEntityUrl(sourceSpace, destinationSpace, url) {
  return url.replace(/api.contentful/, 'app.contentful').replace(/:443/, '').replace(destinationSpace, sourceSpace).split('/').splice(0, 7).join('/');
}

function additionalInfoForUnresolvedLinks(errors) {
  if ((0, _collection.find)(errors, { name: 'UnresolvedLinks' })) {
    return 'Unresolved links were found in your entries. See the errors list for more details. ' + 'Look at https://github.com/contentful/contentful-link-cleaner if you\'d like ' + 'a quicker way to fix all unresolved links in your space.';
  }
}