'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = runSpaceSync;

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _npmlog = require('npmlog');

var _npmlog2 = _interopRequireDefault(_npmlog);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _collection = require('lodash/collection');

var _createClients = require('contentful-batch-libs/utils/create-clients');

var _createClients2 = _interopRequireDefault(_createClients);

var _getSourceSpaceViaSync = require('contentful-batch-libs/get/get-source-space-via-sync');

var _getSourceSpaceViaSync2 = _interopRequireDefault(_getSourceSpaceViaSync);

var _transformSpace = require('contentful-batch-libs/transform/transform-space');

var _transformSpace2 = _interopRequireDefault(_transformSpace);

var _pushToSpace = require('contentful-batch-libs/push/push-to-space');

var _pushToSpace2 = _interopRequireDefault(_pushToSpace);

var _dumpErrorBuffer = require('./dump-error-buffer');

var _dumpErrorBuffer2 = _interopRequireDefault(_dumpErrorBuffer);

var _getTransformedDestinationResponse = require('./get-transformed-destination-response');

var _getTransformedDestinationResponse2 = _interopRequireDefault(_getTransformedDestinationResponse);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

_bluebird2.default.promisifyAll(_fs2.default);

function runSpaceSync(usageParams) {
  var opts = usageParams.opts,
      syncTokenFile = usageParams.syncTokenFile,
      errorLogFile = usageParams.errorLogFile;

  if (opts.proxyHost && opts.proxyPort) {
    opts.proxy = { host: opts.proxyHost, port: opts.proxyPort };
  }
  var clients = (0, _createClients2.default)(opts);
  return (0, _getSourceSpaceViaSync2.default)({
    deliveryClient: clients.source.delivery,
    managementClient: clients.source.management,
    sourceSpaceId: clients.source.spaceId,
    nextSyncTokenFile: syncTokenFile
  })

  // Prepare object with both source and destination existing content
  .then(function (sourceResponse) {
    return _bluebird2.default.props({
      source: sourceResponse,
      destination: (0, _getTransformedDestinationResponse2.default)({
        managementClient: clients.destination.management,
        spaceId: clients.destination.spaceId,
        sourceResponse: sourceResponse,
        skipLocales: opts.skipLocales,
        skipContentModel: opts.skipContentModel
      })
    });
  }).then(function (responses) {
    return _bluebird2.default.props({
      source: (0, _transformSpace2.default)(responses.source, responses.destination),
      destination: responses.destination
    });
  })

  // Get deleted content types
  .then(function (responses) {
    responses.source.deletedContentTypes = (0, _collection.filter)(responses.destination.contentTypes, function (contentType) {
      return !(0, _collection.find)(responses.source.contentTypes, { original: { sys: { id: contentType.sys.id } } });
    });
    responses.source.deletedLocales = (0, _collection.filter)(responses.destination.locales, function (locale) {
      return !(0, _collection.find)(responses.source.locales, { original: { code: locale.code } });
    });
    return responses;
  })

  // push source space content to destination space
  .then(function (responses) {
    return (0, _pushToSpace2.default)({
      sourceContent: responses.source,
      destinationContent: responses.destination,
      managementClient: clients.destination.management,
      spaceId: clients.destination.spaceId,
      prePublishDelay: opts.prePublishDelay,
      contentModelOnly: opts.contentModelOnly,
      skipLocales: opts.skipLocales,
      skipContentModel: opts.skipContentModel
    }).then(function () {
      var nextSyncToken = responses.source.nextSyncToken;
      if (!opts.contentModelOnly && nextSyncToken) {
        _fs2.default.writeFileSync(syncTokenFile, nextSyncToken);
        _npmlog2.default.info('Successfully sychronized the content and saved the sync token to:\n ', syncTokenFile);
      } else {
        _npmlog2.default.info('Successfully sychronized the content model');
      }
      (0, _dumpErrorBuffer2.default)({
        destinationSpace: opts.destinationSpace,
        sourceSpace: opts.sourceSpace,
        errorLogFile: errorLogFile
      }, 'However, additional errors were found');

      return {
        nextSyncToken: nextSyncToken
      };
    });
  })

  // Output any errors caught along the way
  .catch(function (err) {
    (0, _dumpErrorBuffer2.default)({
      destinationSpace: opts.destinationSpace,
      sourceSpace: opts.sourceSpace,
      errorLogFile: errorLogFile
    });
    throw err;
  });
}